package jmeter.robot.wincalculator;

import java.awt.Robot;
import java.awt.AWTException;
import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.ClipboardOwner;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.StringSelection;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;

import java.awt.event.KeyEvent;
import java.io.IOException;

import org.apache.jmeter.config.Arguments;
import org.apache.jmeter.protocol.java.sampler.AbstractJavaSamplerClient;
import org.apache.jmeter.protocol.java.sampler.JavaSamplerContext;
import org.apache.jmeter.samplers.SampleResult;

public class RobotWinCalculator extends AbstractJavaSamplerClient implements ClipboardOwner {

	private static Robot r;

	//Create the Robot
	public RobotWinCalculator() {
		try {
			r = new Robot();
		} catch (AWTException e) {
			e.printStackTrace();
		}
	}

	public Arguments getDefaultParameters() {
		Arguments defaultParameters = new Arguments();
		defaultParameters.addArgument("Number1", "2");
		defaultParameters.addArgument("Number2", "3");
		return defaultParameters;
	}

	@Override
	public SampleResult runTest(JavaSamplerContext context) {

		String Number1 = (context.getParameter("Number1").toString());
		String Number2 = (context.getParameter("Number2").toString());

		RobotWinCalculator rwc = new RobotWinCalculator();

		// run Windows Calculator using command prompt
		r.keyPress(KeyEvent.VK_WINDOWS);
		r.keyPress(KeyEvent.VK_R);
		r.keyRelease(KeyEvent.VK_R);
		r.keyRelease(KeyEvent.VK_WINDOWS);
		// wait for 1000ms
		r.delay(5000);
		// type "calc" and press Enter - calculator is opened
		r.keyPress(KeyEvent.VK_C);
		r.keyRelease(KeyEvent.VK_C);
		r.keyPress(KeyEvent.VK_A);
		r.keyRelease(KeyEvent.VK_A);
		r.keyPress(KeyEvent.VK_L);
		r.keyRelease(KeyEvent.VK_L);
		r.keyPress(KeyEvent.VK_C);
		r.keyRelease(KeyEvent.VK_C);
		r.delay(1000);
		r.keyPress(KeyEvent.VK_ENTER);
		r.keyRelease(KeyEvent.VK_ENTER);
		// wait for 1000ms
		r.delay(1000);

		// Get Number1 test parameter from clipboard
		rwc.setClipboardContents(Number1);
		//Paste Number1 in calculator
		r.keyPress(KeyEvent.VK_CONTROL);
		r.keyPress(KeyEvent.VK_V);
		r.keyRelease(KeyEvent.VK_V);
		r.keyRelease(KeyEvent.VK_CONTROL);
		r.delay(1000);
		
		// Press "+"
		r.keyPress(107);
		r.keyRelease(107);
		r.delay(1000);
		
		// Get Number2 test parameter from clipboard
		rwc.setClipboardContents(Number2);
		//Paste Number2 in calculator
		r.keyPress(KeyEvent.VK_CONTROL);
		r.keyPress(KeyEvent.VK_V);
		r.keyRelease(KeyEvent.VK_V);
		r.keyRelease(KeyEvent.VK_CONTROL);
		r.delay(1000);
		//Press Enter
		r.keyPress(KeyEvent.VK_ENTER);
		r.keyRelease(KeyEvent.VK_ENTER);
		r.delay(1000);
		//Copy result in clipboard
		r.keyPress(KeyEvent.VK_CONTROL);
		r.keyPress(KeyEvent.VK_C);
		r.keyRelease(KeyEvent.VK_C);
		r.keyRelease(KeyEvent.VK_CONTROL);
		r.delay(1000);
		//Get result from clipboard
		String resultClipboard = rwc.getClipboardContents();
		
		// Close calculator
		r.keyPress(KeyEvent.VK_ALT);
		r.keyPress(KeyEvent.VK_F4);
		r.keyRelease(KeyEvent.VK_F4);
		r.keyRelease(KeyEvent.VK_ALT);

		SampleResult result = new SampleResult();
		result.setSuccessful(true);
		result.setResponseMessage(resultClipboard);
		result.setResponseData(resultClipboard.getBytes());
		result.setResponseCodeOK();
		return result;
	}

	public void setClipboardContents(String aString) {
		StringSelection stringSelection = new StringSelection(aString);
		Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
		clipboard.setContents(stringSelection, (ClipboardOwner) this);
	}

	public String getClipboardContents() {
		String result = "";
		Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
		java.awt.datatransfer.Transferable contents = clipboard.getContents(null);
		boolean hasTransferableText = (contents != null) && contents.isDataFlavorSupported(DataFlavor.stringFlavor);
		if (hasTransferableText) {
			try {
				result = (String) contents.getTransferData(DataFlavor.stringFlavor);
			} catch (UnsupportedFlavorException | IOException ex) {
				System.out.println(ex);
				ex.printStackTrace();
			}
		} else
			System.out.println("No data is transffered");
		return result;
	}

	@Override
	public void lostOwnership(Clipboard clipboard, Transferable contents) {

	}

}
